#!/bin/bash

if [ $# -eq 6 ]; then
    localip=$1
    dns=$2
    ipsec_secrets=$3
    listen_iface=$4
    listen_addr=$5
    enabled=$6
elif [ $# -eq 1 -a -f "$1" ]; then
    set -e
    config=$(cat "$1")
    localip=$(echo $config | jq -r '.local_ip')
    dns=$(echo $config | jq -r '.dns')
    ipsec_secrets=$(echo $config | jq -r '.ipsec_secrets')
    listen_iface=$(echo $config | jq -r '.listen_iface')
    listen_addr=$(echo $config | jq -r '.listen_addr')
    enabled=$(echo $config | jq -r '.enabled')
    set +e
else
    echo
    echo "Usage: $0 <ip> <dns> <secrets> <iface> <addr> <enabled>"
    echo "       $0 <file>"
    echo
    exit 1
fi

exiterr()  { echo "Error: $1" >&2; exit 1; }

check_localip() {
  IP_REGEX='^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}1$'
  printf '%s' "$1" | tr -d '\n' | grep -Eq "$IP_REGEX"
}

check_ip() {
  IP_REGEX='^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$'
  printf '%s' "$1" | tr -d '\n' | grep -Eq "$IP_REGEX"
}

check_ipsec_secrets() {
    [ -z "$ipsec_secrets" ] && exiterr "Invalid ipsec secret: '$ipsec_secrets'"
    if printf '%s' "$ipsec_secrets" | LC_ALL=C grep -q '[^ -~]\+'; then
        exiterr "Ipsec secrets must not contain non-ASCII characters: '$ipsec_secrets'"
    fi
    case "$ipsec_secrets" in
        *[\\\"\']*)
        exiterr "Ipsec secrets must not contain [\\ \" ']: '$ipsec_secrets'"
        ;;
    esac
}

check_params() {
    if ! check_localip "$localip"; then
        exiterr "Invalid local ip: '$localip'"
    fi
    if ! check_ip "$dns"; then
        exiterr "Invalid dns: '$dns'"
    fi
    check_ipsec_secrets
    [ -n "$listen_iface" ] || exiterr "Invalid iface: '$listen_iface'"
    if ! check_ip "$listen_addr"; then
        exiterr "Invalid ip address: '$listen_addr'"
    fi
    [ "$enabled" -eq "0" ] || [ "$enabled" -eq "1" ] || exiterr "Enabled must be 0 or 1: '$enabled'"
}

gen_params() {
    seg=${localip%.*}
    l2tp_net="${seg}.0/24"
    l2tp_pool="${seg}.10-${seg}.250"
    if [ "${seg##*.}" -eq "254" ]; then
        seg="${seg%.*}.253"
    else
        seg="${seg%.*}.$((${seg##*.}+1))"
    fi
    xauth_net="${seg}.0/24"
    xauth_pool="${seg}.10-${seg}.250"
}

echo_params() {
    echo
    echo "localip:       '$localip'"
    echo "dns:           '$dns'"
    echo "ipsec_secrets: '$ipsec_secrets'"
    echo "listen_iface:  '$listen_iface'"
    echo "listen_addr:   '$listen_addr'"
    echo "enabled:       '$enabled'"
    echo
}

echo_gen_params() {
    echo "l2tp_net:      '$l2tp_net'"
    echo "l2tp_pool:     '$l2tp_pool'"
    echo "xauth_net:     '$xauth_net'"
    echo "xauth_pool:    '$xauth_pool'"
    echo
}

echo_params
check_params
gen_params
echo_gen_params

/unas/sbin/libreswan/config_l2tp.sh stop
if [ "$enabled" -eq "1" ]; then
    export VPN_L2TP_LOCAL=$localip
    export VPN_DNS_SRV1=$dns
    export VPN_IPSEC_PSK=$ipsec_secrets
    export NET_IFACE=$listen_iface
    export PUBLIC_IP=$listen_addr
    export VPN_L2TP_NET=$l2tp_net
    export VPN_L2TP_POOL=$l2tp_pool
    export VPN_XAUTH_NET=$xauth_net
    export VPN_XAUTH_POOL=$xauth_pool

    /unas/sbin/libreswan/config_l2tp.sh start
fi
